@@ -27,7 +27,7 @@ void usage()
     printf("  -p <phase> : phase offset,              default: 2.8\n");
     printf("  -n <num>   : number of samples,         default: 2400\n");
     printf("  -S <snr>   : SNR [dB],                  default: 30\n");
-    printf("  -t <type>  : AM type (dsb/usb/lsb),     default: dsb\n");
+    printf("  -t <type>  : AM type (dsb/usb/lsb),     default: usb\n");
     printf("  -s         : suppress the carrier,      default: off (carrier enabled)\n");
 }
 
@@ -85,7 +85,7 @@ int main(int argc, char*argv[])
     for (i=0; i<num_samples; i++) {
         x[i] =  0.6f*cos(2*M_PI*0.0202*i);
         x[i] += 0.4f*cos(2*M_PI*0.0271*i);
-        x[i] *= i < nw ? liquid_rcostaper_windowf(i,nt,nw) : 0;
+        x[i] *= i < nw ? liquid_rcostaper_window(i,nw,nt) : 0;
     }
 
     // modulate signal
