@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2017 Joseph Gaeddert
+ * Copyright (c) 2007 - 2020 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +38,7 @@
 #define DEBUG_FLEXFRAMEGEN 1
 
 // reconfigure internal properties
-void          flexframegen_reconfigure      (flexframegen _q);
+int           flexframegen_reconfigure      (flexframegen _q);
 float complex flexframegen_generate_symbol  (flexframegen _q);
 float complex flexframegen_generate_preamble(flexframegen _q);
 float complex flexframegen_generate_header  (flexframegen _q);
@@ -60,9 +60,10 @@ static flexframegenprops_s flexframegenprops_header_default = {
    FLEXFRAME_H_MOD,
 };
 
-void flexframegenprops_init_default(flexframegenprops_s * _props)
+int flexframegenprops_init_default(flexframegenprops_s * _props)
 {
     memmove(_props, &flexframegenprops_default, sizeof(flexframegenprops_s));
+    return LIQUID_OK;
 }
 
 struct flexframegen_s {
@@ -154,7 +155,7 @@ flexframegen flexframegen_create(flexframegenprops_s * _fgprops)
     return q;
 }
 
-void flexframegen_destroy(flexframegen _q)
+int flexframegen_destroy(flexframegen _q)
 {
     // destroy internal objects
     firinterp_crcf_destroy(_q->interp);
@@ -171,10 +172,11 @@ void flexframegen_destroy(flexframegen _q)
 
     // destroy frame generator
     free(_q);
+    return LIQUID_OK;
 }
 
 // print flexframegen object internals
-void flexframegen_print(flexframegen _q)
+int flexframegen_print(flexframegen _q)
 {
     unsigned int num_frame_symbols =
             64 +                    // preamble p/n sequence length
@@ -196,10 +198,11 @@ void flexframegen_print(flexframegen _q)
     printf("  tail          : %u symbols\n", _q->m);
     printf("  total         : %u symbols\n", num_frame_symbols);
     printf("  efficiency    : %.2f bits/second/Hz\n", eta);
+    return LIQUID_OK;
 }
 
 // reset flexframegen object internals
-void flexframegen_reset(flexframegen _q)
+int flexframegen_reset(flexframegen _q)
 {
     // reset internal counters and state
     _q->symbol_counter  = 0;
@@ -207,6 +210,7 @@ void flexframegen_reset(flexframegen _q)
     _q->frame_assembled = 0;
     _q->frame_complete  = 0;
     _q->state           = STATE_PREAMBLE;
+    return LIQUID_OK;
 }
 
 // is frame assembled?
@@ -218,11 +222,12 @@ int flexframegen_is_assembled(flexframegen _q)
 // get flexframegen properties
 //  _q      :   frame generator object
 //  _props  :   frame generator properties structure pointer
-void flexframegen_getprops(flexframegen          _q,
+int flexframegen_getprops(flexframegen          _q,
                            flexframegenprops_s * _props)
 {
     // copy properties structure to output pointer
     memmove(_props, &_q->props, sizeof(flexframegenprops_s));
+    return LIQUID_OK;
 }
 
 // set flexframegen properties
@@ -232,28 +237,20 @@ int flexframegen_setprops(flexframegen          _q,
                           flexframegenprops_s * _props)
 {
     // if frame is already assembled, give warning
-    if (_q->frame_assembled) {
-        fprintf(stderr, "warning: flexframegen_setprops(), frame is already assembled; must reset() first\n");
-        return -1;
-    }
+    if (_q->frame_assembled)
+        return liquid_error(LIQUID_EICONFIG,"flexframegen_setprops(), frame is already assembled; must reset() first");
 
     // if properties object is NULL, initialize with defaults
-    if (_props == NULL) {
-        flexframegen_setprops(_q, &flexframegenprops_default);
-        return 0;
-    }
+    if (_props == NULL)
+        return flexframegen_setprops(_q, &flexframegenprops_default);
 
     // validate input
-    if (_props->check == LIQUID_CRC_UNKNOWN || _props->check >= LIQUID_CRC_NUM_SCHEMES) {
-        fprintf(stderr, "error: flexframegen_setprops(), invalid/unsupported CRC scheme\n");
-        exit(1);
-    } else if (_props->fec0 == LIQUID_FEC_UNKNOWN || _props->fec1 == LIQUID_FEC_UNKNOWN) {
-        fprintf(stderr, "error: flexframegen_setprops(), invalid/unsupported FEC scheme\n");
-        exit(1);
-    } else if (_props->mod_scheme == LIQUID_MODEM_UNKNOWN ) {
-        fprintf(stderr, "error: flexframegen_setprops(), invalid/unsupported modulation scheme\n");
-        exit(1);
-    }
+    if (_props->check == LIQUID_CRC_UNKNOWN || _props->check >= LIQUID_CRC_NUM_SCHEMES)
+        return liquid_error(LIQUID_EICONFIG,"flexframegen_setprops(), invalid/unsupported CRC scheme");
+    if (_props->fec0 == LIQUID_FEC_UNKNOWN || _props->fec1 == LIQUID_FEC_UNKNOWN)
+        return liquid_error(LIQUID_EICONFIG,"flexframegen_setprops(), invalid/unsupported FEC scheme");
+    if (_props->mod_scheme == LIQUID_MODEM_UNKNOWN )
+        return liquid_error(LIQUID_EICONFIG,"flexframegen_setprops(), invalid/unsupported modulation scheme");
 
     // TODO : determine if re-configuration is necessary
 
@@ -261,19 +258,15 @@ int flexframegen_setprops(flexframegen          _q,
     memmove(&_q->props, _props, sizeof(flexframegenprops_s));
 
     // reconfigure payload buffers (reallocate as necessary)
-    flexframegen_reconfigure(_q);
-
-    return 0;
+    return flexframegen_reconfigure(_q);
 }
 
-void flexframegen_set_header_len(flexframegen   _q,
+int flexframegen_set_header_len(flexframegen   _q,
                                  unsigned int   _len)
 {
     // if frame is already assembled, give warning
-    if (_q->frame_assembled) {
-        fprintf(stderr, "warning: flexframegen_setprops(), frame is already assembled; must reset() first\n");
-        return;
-    }
+    if (_q->frame_assembled)
+        return liquid_error(LIQUID_EICONFIG,"flexframegen_setprops(), frame is already assembled; must reset() first");
 
     _q->header_user_len = _len;
     _q->header_dec_len = FLEXFRAME_H_DEC + _q->header_user_len;
@@ -300,47 +293,39 @@ void flexframegen_set_header_len(flexframegen   _q,
     _q->header_sym_len  = qpilotgen_get_frame_len(_q->header_pilotgen);
     _q->header_sym      = (float complex *) realloc(_q->header_sym, _q->header_sym_len*sizeof(float complex));
     //printf("header: %u bytes > %u mod > %u sym\n", 64, _q->header_mod_len, _q->header_sym_len);
+    return LIQUID_OK;
 }
 
 int flexframegen_set_header_props(flexframegen          _q,
                                   flexframegenprops_s * _props)
 {
     // if frame is already assembled, give warning
-    if (_q->frame_assembled) {
-        fprintf(stderr, "warning: flexframegen_set_header_props(), frame is already assembled; must reset() first\n");
-        return -1;
-    }
+    if (_q->frame_assembled)
+        return liquid_error(LIQUID_EICONFIG,"flexframegen_set_header_props(), frame is already assembled; must reset() first");
 
-    if (_props == NULL) {
+    if (_props == NULL)
         _props = &flexframegenprops_header_default;
-    }
 
     // validate input
-    if (_props->check == LIQUID_CRC_UNKNOWN || _props->check >= LIQUID_CRC_NUM_SCHEMES) {
-        fprintf(stderr, "error: flexframegen_set_header_props(), invalid/unsupported CRC scheme\n");
-        exit(1);
-    } else if (_props->fec0 == LIQUID_FEC_UNKNOWN || _props->fec1 == LIQUID_FEC_UNKNOWN) {
-        fprintf(stderr, "error: flexframegen_set_header_props(), invalid/unsupported FEC scheme\n");
-        exit(1);
-    } else if (_props->mod_scheme == LIQUID_MODEM_UNKNOWN ) {
-        fprintf(stderr, "error: flexframegen_set_header_props(), invalid/unsupported modulation scheme\n");
-        exit(1);
-    }
+    if (_props->check == LIQUID_CRC_UNKNOWN || _props->check >= LIQUID_CRC_NUM_SCHEMES)
+        return liquid_error(LIQUID_EIMODE,"flexframegen_set_header_props(), invalid/unsupported CRC scheme\n");
+    if (_props->fec0 == LIQUID_FEC_UNKNOWN || _props->fec1 == LIQUID_FEC_UNKNOWN)
+        return liquid_error(LIQUID_EIMODE,"flexframegen_set_header_props(), invalid/unsupported FEC scheme\n");
+    if (_props->mod_scheme == LIQUID_MODEM_UNKNOWN )
+        return liquid_error(LIQUID_EIMODE,"flexframegen_set_header_props(), invalid/unsupported modulation scheme\n");
 
     // copy properties to internal structure
     memmove(&_q->header_props, _props, sizeof(flexframegenprops_s));
 
     // reconfigure payload buffers (reallocate as necessary)
-    flexframegen_set_header_len(_q, _q->header_user_len);
-
-    return 0;
+    return flexframegen_set_header_len(_q, _q->header_user_len);
 }
 
 // get frame length (number of samples)
 unsigned int flexframegen_getframelen(flexframegen _q)
 {
     if (!_q->frame_assembled) {
-        fprintf(stderr,"warning: flexframegen_getframelen(), frame not assembled!\n");
+        liquid_error(LIQUID_EICONFIG,"flexframegen_getframelen(), frame not assembled!");
         return 0;
     }
     unsigned int num_frame_symbols =
@@ -357,10 +342,10 @@ unsigned int flexframegen_getframelen(flexframegen _q)
 //  _header         :   user-defined header
 //  _payload        :   variable payload buffer (configured by setprops method)
 //  _payload_dec_len:   length of payload
-void flexframegen_assemble(flexframegen          _q,
-                           const unsigned char * _header,
-                           const unsigned char * _payload,
-                           unsigned int          _payload_dec_len)
+int flexframegen_assemble(flexframegen          _q,
+                          const unsigned char * _header,
+                          const unsigned char * _payload,
+                          unsigned int          _payload_dec_len)
 {
     // reset object
     flexframegen_reset(_q);
@@ -409,6 +394,7 @@ void flexframegen_assemble(flexframegen          _q,
 
     // set assembled flag
     _q->frame_assembled = 1;
+    return LIQUID_OK;
 }
 
 // write samples of assembled frame, two samples at a time, returning
@@ -447,7 +433,7 @@ int flexframegen_write_samples(flexframegen    _q,
 //
 
 // reconfigure internal buffers, objects, etc.
-void flexframegen_reconfigure(flexframegen _q)
+int flexframegen_reconfigure(flexframegen _q)
 {
     // configure payload encoder/modulator
     qpacketmodem_configure(_q->payload_encoder,
@@ -463,10 +449,10 @@ void flexframegen_reconfigure(flexframegen _q)
                                                _q->payload_sym_len*sizeof(float complex));
 
     // ensure payload was reallocated appropriately
-    if (_q->payload_sym == NULL) {
-        fprintf(stderr,"error: flexframegen_reconfigure(), could not re-allocate payload array\n");
-        exit(1);
-    }
+    if (_q->payload_sym == NULL)
+        return liquid_error(LIQUID_EIMEM,"flexframegen_reconfigure(), could not re-allocate payload array");
+
+    return LIQUID_OK;
 }
 
 // fill interpolator buffer
@@ -477,13 +463,12 @@ float complex flexframegen_generate_symbol(flexframegen _q)
         return 0.0f;
 
     switch (_q->state) {
-    case STATE_PREAMBLE: return flexframegen_generate_preamble(_q); break;
-    case STATE_HEADER:   return flexframegen_generate_header  (_q); break;
-    case STATE_PAYLOAD:  return flexframegen_generate_payload (_q); break;
-    case STATE_TAIL:     return flexframegen_generate_tail    (_q); break;
+    case STATE_PREAMBLE: return flexframegen_generate_preamble(_q);
+    case STATE_HEADER:   return flexframegen_generate_header  (_q);
+    case STATE_PAYLOAD:  return flexframegen_generate_payload (_q);
+    case STATE_TAIL:     return flexframegen_generate_tail    (_q);
     default:
-        fprintf(stderr,"error: flexframegen_generate_symbol(), unknown/unsupported internal state\n");
-        exit(1);
+        liquid_error(LIQUID_EICONFIG,"flexframegen_generate_symbol(), unknown/unsupported internal state");
     }
 
     return 0.0f;
