@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2022 Joseph Gaeddert
+ * Copyright (c) 2007 - 2023 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -20,11 +20,7 @@
  * THE SOFTWARE.
  */
 
-//
-// ofdmframesync.c
-//
 // OFDM frame synchronizer
-//
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -34,13 +30,55 @@
 
 #include "liquid.internal.h"
 
-#define DEBUG_OFDMFRAMESYNC             1
+#define DEBUG_OFDMFRAMESYNC             0
 #define DEBUG_OFDMFRAMESYNC_PRINT       0
 #define DEBUG_OFDMFRAMESYNC_FILENAME    "ofdmframesync_internal_debug.m"
 #define DEBUG_OFDMFRAMESYNC_BUFFER_LEN  (2048)
 
 #define OFDMFRAMESYNC_ENABLE_SQUELCH    0
 
+// forward declaration of internal methods
+
+int ofdmframesync_execute_seekplcp(ofdmframesync _q);
+int ofdmframesync_execute_S0a(ofdmframesync _q);
+int ofdmframesync_execute_S0b(ofdmframesync _q);
+int ofdmframesync_execute_S1( ofdmframesync _q);
+int ofdmframesync_execute_rxsymbols(ofdmframesync _q);
+
+int ofdmframesync_S0_metrics(ofdmframesync   _q,
+                             float complex * _G,
+                             float complex * _s_hat);
+
+// estimate short sequence gain
+//  _q      :   ofdmframesync object
+//  _x      :   input array (time)
+//  _G      :   output gain (freq)
+int ofdmframesync_estimate_gain_S0(ofdmframesync   _q,
+                                   float complex * _x,
+                                   float complex * _G);
+
+// estimate long sequence gain
+//  _q      :   ofdmframesync object
+//  _x      :   input array (time)
+//  _G      :   output gain (freq)
+int ofdmframesync_estimate_gain_S1(ofdmframesync _q,
+                                   float complex * _x,
+                                   float complex * _G);
+
+// estimate complex equalizer gain from G0 and G1
+//  _q      :   ofdmframesync object
+//  _ntaps  :   number of time-domain taps for smoothing
+//int ofdmframesync_estimate_eqgain(ofdmframesync _q, unsigned int _ntaps);
+
+// estimate complex equalizer gain from G0 and G1 using polynomial fit
+//  _q      :   ofdmframesync object
+//  _order  :   polynomial order
+int ofdmframesync_estimate_eqgain_poly(ofdmframesync _q,
+                                       unsigned int _order);
+
+// recover symbol, correcting for gain, pilot phase, etc.
+int ofdmframesync_rxsymbol(ofdmframesync _q);
+
 struct ofdmframesync_s {
     unsigned int M;         // number of subcarriers
     unsigned int M2;        // number of subcarriers (divided by 2)
@@ -149,11 +187,13 @@ ofdmframesync ofdmframesync_create(unsigned int           _M,
 
     // validate input
     if (_M < 8)
-        return liquid_error_config("ofdmframesync_create(), less than 8 subcarriers");
+        return liquid_error_config("ofdmframesync_create(), number of subcarriers must be at least 8");
     if (_M % 2)
         return liquid_error_config("ofdmframesync_create(), number of subcarriers must be even");
     if (_cp_len > _M)
         return liquid_error_config("ofdmframesync_create(), cyclic prefix length cannot exceed number of subcarriers");
+    if (_taper_len > _cp_len)
+        return liquid_error_config("ofdmframesync_create(), taper length cannot exceed cyclic prefix");
 
     q->M = _M;
     q->cp_len = _cp_len;
@@ -172,12 +212,6 @@ ofdmframesync ofdmframesync_create(unsigned int           _M,
     // validate and count subcarrier allocation
     if (ofdmframe_validate_sctype(q->p, q->M, &q->M_null, &q->M_pilot, &q->M_data))
         return liquid_error_config("ofdmframesync_create(), invalid subcarrier allocation");
-    if ( (q->M_pilot + q->M_data) == 0)
-        return liquid_error_config("ofdmframesync_create(), must have at least one enabled subcarrier");
-    if (q->M_data == 0)
-        return liquid_error_config("ofdmframesync_create(), must have at least one data subcarriers");
-    if (q->M_pilot < 2)
-        return liquid_error_config("ofdmframesync_create(), must have at least two pilot subcarriers");
 
     // create transform object
     q->X = (float complex*) FFT_MALLOC((q->M)*sizeof(float complex));
@@ -417,8 +451,7 @@ float ofdmframesync_get_cfo(ofdmframesync _q)
 // set receiver carrier frequency offset estimate
 int ofdmframesync_set_cfo(ofdmframesync _q, float _cfo)
 {
-    nco_crcf_set_frequency(_q->nco_rx, _cfo);
-    return LIQUID_OK;
+    return nco_crcf_set_frequency(_q->nco_rx, _cfo);
 }
 
 //
@@ -757,7 +790,7 @@ int ofdmframesync_execute_rxsymbols(ofdmframesync _q)
 }
 
 // compute S0 metrics
-int ofdmframesync_S0_metrics(ofdmframesync _q,
+int ofdmframesync_S0_metrics(ofdmframesync   _q,
                              float complex * _G,
                              float complex * _s_hat)
 {
@@ -845,6 +878,7 @@ int ofdmframesync_estimate_gain_S1(ofdmframesync _q,
     return LIQUID_OK;
 }
 
+#if 0
 // estimate complex equalizer gain from G0 and G1
 //  _q      :   ofdmframesync object
 //  _ntaps  :   number of time-domain taps for smoothing
@@ -905,6 +939,7 @@ int ofdmframesync_estimate_eqgain(ofdmframesync _q,
     }
     return LIQUID_OK;
 }
+#endif
 
 // estimate complex equalizer gain from G0 and G1 using polynomial fit
 //  _q      :   ofdmframesync object
@@ -955,12 +990,7 @@ int ofdmframesync_estimate_eqgain_poly(ofdmframesync _q,
         return liquid_error(LIQUID_EINT,"ofdmframesync_estimate_eqgain_poly(), pilot subcarrier mismatch");
 
     // try to unwrap phase
-    for (i=1; i<N; i++) {
-        while ((y_arg[i] - y_arg[i-1]) >  M_PI)
-            y_arg[i] -= 2*M_PI;
-        while ((y_arg[i] - y_arg[i-1]) < -M_PI)
-            y_arg[i] += 2*M_PI;
-    }
+    liquid_unwrap_phase(y_arg, N);
 
     // fit to polynomial
     polyf_fit(x_freq, y_abs, N, p_abs, _order+1);
@@ -1036,12 +1066,7 @@ int ofdmframesync_rxsymbol(ofdmframesync _q)
         return liquid_error(LIQUID_EINT,"ofdmframesync_estimate_eqgain_poly(), pilot subcarrier mismatch");
 
     // try to unwrap phase
-    for (i=1; i<_q->M_pilot; i++) {
-        while ((y_phase[i] - y_phase[i-1]) >  M_PI)
-            y_phase[i] -= 2*M_PI;
-        while ((y_phase[i] - y_phase[i-1]) < -M_PI)
-            y_phase[i] += 2*M_PI;
-    }
+    liquid_unwrap_phase(y_phase, _q->M_pilot);
 
     // fit phase to 1st-order polynomial (2 coefficients)
     polyf_fit(x_phase, y_phase, _q->M_pilot, p_phase, 2);
@@ -1127,7 +1152,7 @@ int ofdmframesync_debug_enable(ofdmframesync _q)
     _q->debug_objects_created = 1;
     return LIQUID_OK;
 #else
-    fprintf(stderr,"ofdmframesync_debug_enable(): compile-time debugging disabled\n");
+    return liquid_error(LIQUID_EICONFIG,"ofdmframesync_debug_enable(): compile-time debugging disabled");
 #endif
 }
 
@@ -1138,7 +1163,7 @@ int ofdmframesync_debug_disable(ofdmframesync _q)
     _q->debug_enabled = 0;
     return LIQUID_OK;
 #else
-    fprintf(stderr,"ofdmframesync_debug_disable(): compile-time debugging disabled\n");
+    return liquid_error(LIQUID_EICONFIG,"ofdmframesync_debug_disable(): compile-time debugging disabled");
 #endif
 }
 
@@ -1300,7 +1325,7 @@ int ofdmframesync_debug_print(ofdmframesync _q,
     printf("ofdmframesync/debug: results written to '%s'\n", _filename);
     return LIQUID_OK;
 #else
-    fprintf(stderr,"ofdmframesync_debug_print(): compile-time debugging disabled\n");
+    return liquid_error(LIQUID_EICONFIG,"ofdmframesync_debug_print(): compile-time debugging disabled");
 #endif
 }
 
