@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2015 Joseph Gaeddert
+ * Copyright (c) 2007 - 2022 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,5 +33,6 @@
 
 #define T_COMPLEX           1
 
-// source files
-#include "quantizer.c"
+// prototypes
+#include "quantizer.proto.c"
+
